@@ -65,6 +65,12 @@ class Chart {
             mChart->setAxesLimits(pXmin, pXmax, pYmin, pYmax, pZmin, pZmax);
         }
 
+        inline void setAxesLabelFormat(const std::string& pXFormat,
+                                       const std::string& pYFormat,
+                                       const std::string& pZFormat) {
+            mChart->setAxesLabelFormat(pXFormat, pYFormat, pZFormat);
+        }
+
         inline void getAxesLimits(float* pXmin, float* pXmax,
                                   float* pYmin, float* pYmax,
                                   float* pZmin, float* pZmax) {
